
# q-expansion of newform 7225.2.a.j, downloaded from the LMFDB on 16 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7225
weight = 2
poly_data = [-4, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, -1], [0, -1], [0, 0], [2, 1], [3, -1], [-4, 2], [0, 0], [1, 1], [0, -1], [-5, -1], [0, 0], [10, 0], [7, 0], [-6, 2], [-2, 2], [-2, -2], [-1, -1], [-9, -1], [-2, 1], [-9, 3], [-10, -2], [-11, -1], [-6, 1], [5, -8], [-10, 4], [-5, 6], [-6, 5], [-6, -2], [2, 5], [2, -4], [-16, -1], [5, -5], [-8, 2], [15, -1], [-10, 3], [7, -9], [0, 2], [-8, -1], [-2, -6], [-6, -4], [9, 1], [-6, -3], [15, -3], [-2, -2], [0, -10], [-5, 3], [-4, 2], [-6, 3], [8, -9], [5, 2], [0, -4], [-1, 9], [3, -8], [-9, 3], [20, -4], [12, -5], [10, -3], [3, 5], [-4, 0], [-3, 4], [-8, 9], [18, 6], [-8, -4], [20, -2], [-12, 4], [0, 4], [11, -7], [-8, -4], [-6, 6], [9, -2], [2, 10], [13, 7], [8, 0], [-12, 4], [13, -11], [8, -13], [-1, 3], [-2, 14], [9, -4], [-9, 0], [4, -6], [-18, 12], [8, -16], [4, 4], [-17, -1], [18, 3], [-15, -3], [18, -4], [6, -5], [-8, 7], [-4, 5], [-25, -3], [2, -13], [4, -12], [-17, 9], [-16, 3], [-19, -2], [-3, 0], [-10, -5], [5, 2], [-2, 9], [22, 4], [4, 8], [3, 4], [-9, 1], [-4, 16], [8, -19], [-24, 0], [-9, 3], [2, 5], [-32, 4], [-30, 0], [-8, 0], [19, 1], [35, 1], [18, -7], [12, -3], [2, 5], [28, -12], [25, -13], [-14, -7], [24, -4], [16, 10], [-14, -3], [-4, -6], [29, 2], [7, -14], [-3, 5], [-4, 12], [-8, 4], [-8, 2], [34, -7], [8, -12], [24, 8], [-29, -5], [-18, 7], [-24, -10], [-10, -3], [-6, 4], [27, 4], [-13, 19], [6, 11], [-20, -5], [-4, -11], [15, 6], [-43, 3], [10, -16], [-2, 12], [20, 2], [-16, 0], [36, 4], [19, -1], [-16, 8], [-14, -7], [-30, -6], [28, -6], [-48, 0], [-31, 12], [10, -20], [-7, -18], [-6, 5], [20, 8], [-12, -15], [-11, -7], [2, 22], [26, 11], [24, 2], [12, 16], [15, 11], [-8, -2], [40, 8], [-1, 22], [-32, -8], [6, -16], [9, 5], [10, 9], [11, 15], [-15, 10], [2, 5], [-29, 2], [-46, 6], [-36, 10], [44, 8], [6, -8], [18, -23], [6, -20], [-40, 10], [36, -17], [13, -20], [-56, 2], [24, -10], [28, -8], [13, 19], [-7, 14], [46, 1], [30, -6], [-1, 23], [24, -12], [-4, 16], [-10, 1], [-29, 22], [-11, -15], [12, -12], [-2, 4], [-4, 6], [2, 6], [23, -13], [-4, 4], [-5, 16], [-36, 4], [-26, 18], [18, -11], [-4, 4], [-12, -8], [-48, 2], [6, -28], [16, -28], [-45, 15], [-30, -10], [10, 0], [-2, 12], [17, 17], [-10, -13], [-16, 0], [10, 15], [-14, -9], [4, 10], [-37, 5], [-38, -2], [-19, -7], [-2, 6], [21, -33], [12, -2], [9, 13], [-36, 0], [56, 3], [-54, 4], [28, -2], [44, -2], [-36, 8], [-18, -5], [15, -21], [14, -13], [-23, 5], [-22, 4], [-5, 3], [-16, -1], [29, -19], [-15, 21], [30, 3], [18, 12], [-6, -21], [36, -1], [9, 12], [-6, 22], [-29, -1], [-18, 5], [4, 17], [6, -18], [-48, -4], [-58, 5], [10, -3], [-25, 7], [0, 14], [24, 0], [27, 1], [-7, -3], [3, 20], [6, -30], [32, -4], [33, -2], [-16, 5], [30, -12], [-27, 11], [10, 4], [-42, 2], [-16, 13], [-6, 17], [-27, -8], [-28, -12], [-40, 25], [-5, 31], [-58, 15], [-9, -6], [-6, 21], [35, -5], [0, -16], [-24, 14], [-8, 4], [27, -20], [35, -2], [-14, 15], [-38, -6], [41, 15], [-48, 6], [-43, -6], [-51, 15], [14, -30], [12, 28], [50, -9], [42, -26], [-44, 4], [-44, -4], [-4, -24], [-28, 26], [-56, 4], [-22, -19], [3, -25], [-41, 9], [-6, -8], [44, -1], [-37, 2], [37, 0], [32, -21], [4, 31], [25, 0], [-43, 3], [-17, 27], [46, 0], [15, -29], [-32, 0], [40, -20], [46, -16], [-10, -29], [-26, -18], [-58, 15], [-69, 1], [-40, -12], [-10, -27], [2, -12], [14, 3], [-14, -6], [13, 3], [-42, -18], [4, 20], [18, 1], [30, -25], [12, 10], [14, 12], [14, -15], [-6, 36], [46, -24], [61, -10], [-31, -5], [-2, -10], [-1, -15], [23, 22], [-60, 4], [25, 1], [-34, 30], [44, -28], [-44, 20], [-23, -26], [-16, 13], [1, 2], [10, -24], [28, -20], [-87, -1], [38, -12], [-18, -5], [-28, 20], [-55, 21], [26, -25], [-29, 1], [50, 1], [28, -14], [42, -18], [2, 9], [38, -32], [28, 0], [-55, -5], [-28, 33], [-41, 26], [43, -5], [-36, -22], [-55, 1], [-8, -8], [-20, 8], [-13, 32], [0, 20], [25, -3], [52, -16], [-52, 4], [-32, 38], [16, -18], [46, -9], [84, -2], [26, 0], [20, -19], [28, -25], [61, 4], [-20, 10], [-41, 29], [80, -8], [19, 17], [62, 0], [13, 21], [5, 9], [9, -13], [21, -3], [29, -2], [-38, 26], [24, -16], [48, -6], [63, -14], [-27, -9], [-56, 12], [-46, -11], [24, -17], [-8, -4], [-18, -20], [-62, 6], [-12, 3], [35, 5], [48, 20], [13, -30], [11, -19], [-22, 5], [-58, 10], [32, 3], [7, 31], [-32, -2], [82, -1], [3, -41], [64, -2], [-50, -22], [2, 31], [-2, -17], [-4, 8], [5, -13]]
